/*
 * pop_get.c
 * Sample POP3 -> mbox program
 * Links again: libpost_raw
 * Links shared
 * $Id: pop_get.c,v 1.12 2003/07/16 17:18:02 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "../../include/post_raw/post_raw.h"


int
main(int argc, char *argv[])
{
	POSTAL_CONN *pop_conn, *local_conn;
	POSTAL_MSG_GROUP *ret_msgs;
	int i;
	int dele=0;
	int goch;
	int retval;
	char *p_serv=NULL, *p_user=NULL, *p_pass=NULL;
	char *mbox_name=NULL, *maildir_name=NULL;
	char *binfo;

	while ( (goch = getopt(argc, argv, "dh:u:p:m:M:")) != -1 )
		switch((char)goch)
		{
			case 'd':
				/* Do DELE's */
				dele = 1;
				break;
			case 'h':
				/* Hostname */
				p_serv = optarg;
				break;
			case 'u':
				/* Username */
				p_user = optarg;
				break;
			case 'p':
				/* Password */
				p_pass = optarg;
				break;
			case 'm':
				/* mbox filename */
				mbox_name = optarg;
				break;
			case 'M':
				/* Maildir filename */
				maildir_name = optarg;
				break;
			default:
				printf("Bad argument '%c'\n", goch);
				exit(1);
				/* NOTREACHED */
		}

	argc -= optind;
	argv += optind;

	/* Error-check */
	if( ((mbox_name==NULL) && (maildir_name==NULL)) )
	{
		fprintf(stderr, "Either Maildir or mbox must be specified.\n");
		exit(1);
	}

	if( (mbox_name!=NULL) && (maildir_name!=NULL) )
	{
		fprintf(stderr, "Only one of Maildir or mbox may be specified.\n");
		exit(1);
	}

	/* Grab any non-command-line spec'd options interactively */
	if(!p_serv)
	{
		printf("Enter hostname to fetch from: ");
		fflush(stdout);
		p_serv = malloc(1024);
		scanf("%s", p_serv);
	}
	if(!p_user)
	{
		printf("Enter username: ");
		fflush(stdout);
		p_user = malloc(1024);
		scanf("%s", p_user);
	}
	if(!p_pass)
	{
		p_pass = getpass("Enter password: ");
	}


	if(mbox_name)
	{
		binfo = malloc(strlen(mbox_name)+2+5+1);
		sprintf(binfo, "mbox '%s'", mbox_name);
	}
	else if(maildir_name)
	{
		binfo = malloc(strlen(maildir_name)+2+8+1);
		sprintf(binfo, "maildir '%s'", maildir_name);
	}

	printf("Running with host '%s', user '%s', and pass '%s', "
			"writing into %s\n",
			p_serv, p_user, p_pass, binfo);

	pop_conn = pop_connect(p_serv, p_user, p_pass);

	if(!pop_conn)
	{
		printf("You lose, too bad!\n");
		exit(1);
	}

	/* Clean some stuff up */
	bzero(p_pass, strlen(p_pass));

	/* Get all the messages at once with pop_retr_all() */
	ret_msgs = pop_retr_all(pop_conn, dele);
	pop_close(pop_conn);

	if(ret_msgs==NULL)
	{
		fprintf(stderr, "Error POP'ing messages: %s\n",
				postal_strerror(postal_errno()));
		exit(1);
	}

	/* Open up the mbox */
	if(mbox_name)
		local_conn = mbox_open_file(mbox_name, LOCK_DEFAULT, OPEN_READWRITE);
	else if(maildir_name)
		local_conn = maildir_open(maildir_name, 1);

	if(local_conn==NULL)
	{
		fprintf(stderr, "Error opening local %s: %s\n", binfo,
				postal_strerror(postal_errno()));
		exit(1);
	}

	/* Write out the messages */
	for(i=1 ; i <= postal_mgroup_num(ret_msgs) ; i++)
	{
		if(mbox_name)
			retval=mbox_write_msg(local_conn,
					postal_mgroup_getmsg(ret_msgs,i), POS_END);
		else if(maildir_name)
			retval=maildir_write_msg(local_conn,
					postal_mgroup_getmsg(ret_msgs,i));

		if(retval<0)
		{
			fprintf(stderr, "Error writing message %d to %s: %s\n",
					i, binfo, postal_strerror(postal_errno()));
			if(mbox_name)
				mbox_close_file(local_conn);
			else if(maildir_name)
				maildir_close(local_conn);

			exit(1);
		}
	}

	/* Free things up */
	postal_free_msggroup(ret_msgs);

	if(mbox_name)
		mbox_close_file(local_conn);
	else if(maildir_name)
		maildir_close(local_conn);

	/*
	 * XXX We may leak memory on the p_* and binfo variables.  It would
	 * take actually trying to fix that, it's a few bytes, and the
	 * program exits quick anyway.  I'll deal with it in a future
	 * incarnation.
	 */
	exit(0);
}
