/*
 * postal_change_header.c
 * Change the value of a specified header in a POSTAL_MSG
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_change_header.c,v 1.9 2003/06/03 13:30:59 fullermd Exp $
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>


int
postal_change_header(POSTAL_MSG *msg, const char *hdr, const char *content)
{
	char *buf, *buf2, *buf3;
	char *tmpbuf;
	char *tofind;
	int nxtfnd=0;
	size_t nowlen, adjust;

	if(!hdr || !content || !msg || !msg->header)
		POSTAL_ERRSET_RI(POSTAL_E_INVAL, NULL);

	tofind = malloc(strlen(hdr)+2);
	if(tofind==NULL)
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	sprintf(tofind, "\n%s", hdr);

	if(!strncmp(msg->header, hdr, strlen(hdr)))
	{
		/* Got it at the beginning */
		buf=msg->header;
	}
	else
	{
		/* It's somewhere in the middle */
		buf=postal_casestr(msg->header, tofind);
		if(buf)
			buf++;
	}


	/* First, see if the header's even here, bail otherwise */
	if(!buf)
	{
		/* Not found */
		free(tofind);
		POSTAL_ERRSET_RI(POSTAL_E_NOHDR, NULL);
	}

	/* Move forward to the meat */
	/* XXX May want to reconsider whitespace catching */
	buf3=buf;
	buf += strlen(hdr);
	while(*buf==' ')
		buf++;

	/*
	 * Find the next header.
	 * Don't just do a strchr() for \n!  Headers
	 * may span lines!
	 */
	buf2 = strchr(buf, '\n');
	while(nxtfnd==0)
	{
		if(!buf2)
		{
			/*
			 * No \n found.  This probably shouldn't happen.
			 */
			free(tofind);
			POSTAL_ERRSET_RI(POSTAL_E_DFORM,
					"No newline where newline should be in headers");
		}
		else if(*(buf2+1)=='\0')
		{
			/* At the end of the headers */
			nxtfnd=1;
		}
		else if(isalpha(*(buf2+1)))
		{
			/* This is the end of the header */
			nxtfnd=1;
		}
		else
		{
			/* Try again */
			buf2 = strchr(buf2, '\n');
		}
	}

	/* Check our length */
	nowlen = buf2-buf;

	/* Now, do a few hat tricks */
	if(nowlen == strlen(content))
	{
		/* Ha!  They're the same length!  Watch THIS! */
		strcpy(buf, content);
	}
	else
	{
		adjust = strlen(content)-nowlen;

		/* We're going to realloc(), so we can damage the original */
		*buf3='\0';
		buf2++;
		tmpbuf=malloc(msg->hdrlen+adjust+1);
		if(tmpbuf==NULL)
		{
			free(tofind);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}
		sprintf(tmpbuf, "%s%s: %s\n%s", msg->header, hdr, content, buf2);
		free(msg->header);
		msg->header=tmpbuf;
		msg->hdrlen += adjust;
	}

	/* If we got here, we're clean */
	free(tofind);
	return(0);
}
