/*
 * postal_err_foo.c
 * Error-handling functions
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_err_foo.c,v 1.4 2003/06/04 10:01:35 fullermd Exp $
 */

/*
 * These are all being crammed into a single file, rather against
 * established practice for the project.  This is being done for several
 * reasons:
 *   - All of these are going to act differently based on whether we're
 *     compiling in pthreads support, and (hopefully, anyway) nothing
 *     else will, so these are the only things that need to be variantly
 *     compiled.
 *   - I'm going to be doing some really twisted things I'm not proud of
 *     to the compilation process to make it work for that pthreads
 *     variation, and I'd rather have vile hacks for as few files as
 *     possible.
 */

/* This line will be sed(1)-ified in the build process */
#undef __ENABLE_PTHREADS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef __ENABLE_PTHREADS
#include <pthread.h>
#endif

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>


/* Global error variables */
#ifdef __ENABLE_PTHREADS
 pthread_key_t postal_errfoo_key;
#else
 POSTAL_ERRFOO_T postal_errfoo;
#endif


/* Internal-only: Set an error */
void
postal_err_set(int p_errno, const char *p_errstr)
{
	POSTAL_ERRFOO_T *postal_errfoo_p =
#ifdef __ENABLE_PTHREADS
			(POSTAL_ERRFOO_T *)pthread_getspecific(postal_errfoo_key);
#else
			&postal_errfoo;
#endif
	
	postal_errfoo_p->postal_errno = p_errno;
	if(p_errstr!=NULL && (strlen(p_errstr)>0))
	{
		strncpy(postal_errfoo_p->postal_errstr, p_errstr,
				POSTAL_ERRSTR_LEN);
		postal_errfoo_p->postal_errno |= POSTAL_ERRFOO_HASSTR;
	}
	
	return;
}


/* External: Get the error number */
unsigned int
postal_errno(void)
{
	POSTAL_ERRFOO_T *postal_errfoo_p =
#ifdef __ENABLE_PTHREADS
			(POSTAL_ERRFOO_T *)pthread_getspecific(postal_errfoo_key);
#else
			&postal_errfoo;
#endif

	return(((postal_errfoo_p->postal_errno)&~POSTAL_ERRFOO_HASSTR));
}


/* External: Get the error string if it exists */
const char *
postal_errstr(void)
{
	POSTAL_ERRFOO_T *postal_errfoo_p =
#ifdef __ENABLE_PTHREADS
			(POSTAL_ERRFOO_T *)pthread_getspecific(postal_errfoo_key);
#else
			&postal_errfoo;
#endif

	if(postal_errfoo_p->postal_errno & POSTAL_ERRFOO_HASSTR)
		return(postal_errfoo_p->postal_errstr);
	
	return(NULL);
}
