/*
 * postal_fcntl_lock.c
 * fcntl() lock a file
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_fcntl_lock.c,v 1.4 2003/06/03 13:31:00 fullermd Exp $
 */

/* sys/file.h needed on Linux for LOCK_* */
#include <sys/file.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>

/*
 * These should be something other than global vars someday,
 * if we want to ever be threadsafe.
 */

unsigned fcntl_wait=1;
int fcntl_retry=5;

int
postal_fcntl_lock(int tolock, int type)
{
	int count=0;
	struct flock lockit;

	/* Say what we're trying to lock */
	lockit.l_start=0;
	lockit.l_len=0;
	lockit.l_whence=SEEK_SET;
	if(type&LOCK_EX)
		lockit.l_type=F_WRLCK;
	else
		lockit.l_type=F_RDLCK;

	while( fcntl(tolock, F_SETLK, &lockit)!=0 )
	{
		if(errno!=EAGAIN)
			POSTAL_ERRSET_RI(POSTAL_E_LKFAIL, NULL);
		
		/*
		 * EAGAIN implies that somebody else already holds the lock.
		 * Wait and try again, unless we've already exceeded the number
		 * of retries.
		 */
		if(count>=fcntl_retry)
			POSTAL_ERRSET_RI(POSTAL_E_LKFAIL, NULL);

		count++;
		sleep(fcntl_wait);
	}
	
	/* If we got here, fcntl succeeded */
	return(0);
}
