/*
 * postal_flush_line.c
 * Flush to EOL
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_flush_line.c,v 1.5 2003/06/03 13:31:00 fullermd Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>

int
postal_flush_line(int sock)
{
	ssize_t peek_len, readret;
	size_t toread=1024;
	char *ptr, *tmpbuf;

	/* XXX  This can be a tight loop eating CPU with a slow server */
	while( (!ptr) )
	{
		/* PEEK as much as we can out of the socket queue */
		peek_len = recv(sock, tmpbuf, toread, MSG_PEEK);
		if( (peek_len < 0) && (errno != EINTR) )
			POSTAL_ERRSET_RI(POSTAL_E_SOCKET, "recv()");

		/* Look for EOL */
		ptr = memchr( tmpbuf, '\n', peek_len );
		if( ptr != NULL )
		{
			/* XXX This pointer arithmetic might not be portable */
			peek_len = ptr - tmpbuf + 1;
		}
		else
		{
			if(peek_len == toread)
			{
				/* Read more!  God, what a hack */
				toread += 1024;
			}
		}
	}

	/* God, I love dummy buffers :) */
	tmpbuf=malloc(peek_len);
	if(tmpbuf==NULL)
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	readret = read(sock, tmpbuf, peek_len);
	free(tmpbuf);
	if( (readret != peek_len) && (errno != EINTR) )
		POSTAL_ERRSET_RI(POSTAL_E_SOCKET, NULL);

	/* If we got here, things worked */
	return(0);
}
