/*
 * postal_get_header.c
 * Get a specified header from a POSTAL_MSG
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_get_header.c,v 1.13 2003/06/03 13:31:00 fullermd Exp $
 */

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>


char *
postal_get_header(const char *hdrs, const char *toget)
{
	char *toret, *hdrwork, *tofind, *buf, *buf2;
	int nxtfnd=0;

	if(!toget || !hdrs)
		POSTAL_ERRSET_RP(POSTAL_E_INVAL, NULL);

	hdrwork = malloc(strlen(hdrs)+1);
	if(hdrwork==NULL)
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	strcpy(hdrwork, hdrs);
	
	tofind = malloc(strlen(toget)+2);
	if(tofind==NULL)
	{
		free(hdrwork);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	sprintf(tofind, "\n%s", toget);

	if(!strncasecmp(hdrwork, toget, strlen(toget)))
	{
		/* Got it at the beginning */
		buf=hdrwork;
	}
	else
	{
		/* It's somewhere in the middle */
		buf=postal_casestr(hdrwork, tofind);
		if(buf)
			buf++;
	}

	if(buf)
	{
		/*
		 * Header found.  Find the next header.
		 * Don't just do a strchr() for \n!  Headers
		 * may span lines!
		 */
		buf2 = strchr(buf, '\n');
		while(nxtfnd==0)
		{
			if(!buf2)
			{
				/*
				 * No \n found.  This probably shouldn't happen.
				 */
				free(hdrwork);
				free(tofind);
				POSTAL_ERRSET_RP(POSTAL_E_DFORM, NULL);
			}
			else if(*(buf2+1)=='\0')
			{
				/* At the end of the headers */
				nxtfnd=1;
			}
			else if(isalpha(*(buf2+1)))
			{
				/* This is the end of the header */
				*buf2='\0';
				nxtfnd=1;
			}
			else
			{
				/* Try again */
				buf2 = strchr(buf2, '\n');
			}
		}
	}
	else
	{
		/* Header isn't found */
		free(hdrwork);
		free(tofind);
		POSTAL_ERRSET_RP(POSTAL_E_NOERR, NULL);
	}

	/*
	 * We've now got a full null-terminated header at
	 * buf.  Chop off the titular portion of the header
	 * and slap it into the var to return.
	 */
	buf += strlen(toget);
	/* Whack leading spaces */
	while(*buf==' ')
		buf++;
	toret = malloc(strlen(buf)+1);
	if(toret==NULL)
	{
		free(hdrwork);
		free(tofind);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	strcpy(toret, buf);
	free(hdrwork);
	free(tofind);
	return(toret);
}
