/*
 * postal_get_line.c
 * Cleanly read a \r\n-delimited line from a socket
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_get_line.c,v 1.7 2003/06/03 13:31:00 fullermd Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>

/*
 * Read data from a socket, stopping at the first newline
 * or at a given length.
 */
int
postal_get_line(int sock, char *retbuf, unsigned max)
{
	size_t peek_len, read_len;
	int found=0;
	char *ptr, *tmpbuf;

	/* Sanity check and cheat.  This could cause problems. */
	if(max<2)
		max=2;

	tmpbuf = malloc(max);
	if(tmpbuf==NULL)
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);

	/* XXX  This can be a tight loop eating CPU with a slow server */
	while(!found)
	{
		/* PEEK as much as we can out of the socket queue */
		peek_len = recv(sock, tmpbuf, max-1, MSG_PEEK);
		if( peek_len < 0 )
		{
			free(tmpbuf);
			POSTAL_ERRSET_RI(POSTAL_E_SOCKET, NULL);
		}

		/* Look for EOL */
		ptr = memchr( tmpbuf, '\n', peek_len );
		if( ptr != NULL )
		{
			/* XXX This pointer arithmetic might not be portable */
			peek_len = ptr - tmpbuf + 1;
			found=1;
		}
	}

	/*
	 * We shouldn't have to loop or error-check here,
	 * because if we previously managed to PEEK peek_len
	 * bytes, it should be able to read that many.
	 * If that's not true of some implementation, we
	 * may have to stick a loop/check on this.
	 */
	read_len = read(sock, tmpbuf, peek_len);
	if( read_len != peek_len )
	{
		free(tmpbuf);
		POSTAL_ERRSET_RI(POSTAL_E_SOCKET, NULL);
	}

	/* If we have the newline, kill it, else null-term normally */
	if(ptr)
		tmpbuf[(read_len-2)] = '\0';
	else
		tmpbuf[read_len] = '\0';

	/* Copy over for return */
	strcpy(retbuf, tmpbuf);

	/* Cleanup */
	free(tmpbuf);

	/* Return 0 if we ran out of space, otherwise string length */
	return (ptr == NULL) ? 0 : strlen(retbuf);
}
