/*
 * postal_set_header.c
 * Set a header to a specific value
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_set_header.c,v 1.6 2003/06/05 11:19:28 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>


int
postal_set_header(POSTAL_MSG *msg, const char *hdr, const char *content)
{
	char *head;
	char *tmpbuf;

	if(!hdr || !content || !msg || !msg->header)
		POSTAL_ERRSET_RI(POSTAL_E_INVAL, NULL);

	head = postal_get_header(msg->header, hdr);
	if(head)
	{
		/* Already exists, change it */
		postal_change_header(msg, hdr, content);
		free(head);
	}
	else
	{
		if(postal_errno() != POSTAL_E_NOERR)
			POSTAL_ERRRET_RI; /* Preserve errno */
		/* Doesn't exist, create it */
		tmpbuf=malloc(strlen(hdr)+strlen(content)+1+1);
		if(tmpbuf==NULL)
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		sprintf(tmpbuf, "%s %s", hdr, content);
		if(postal_add_header(msg, tmpbuf)<0)
		{
			free(tmpbuf);
			POSTAL_ERRRET_RI; /* Preserve errno */
		}
		free(tmpbuf);
	}
	
	return(0);
}
