/*
 * post_raw_p.h
 * Private header file
 * libpostal project
 * libpost_raw
 * $Id: post_raw_p.h,v 1.26 2003/07/16 11:41:14 fullermd Exp $
 */

#ifndef _LIBPOSTAL_POST_RAW_P_H_
#define _LIBPOSTAL_POST_RAW_P_H_

/* #include's in #includes, bad boy! */
#include <stdio.h>
#include <string.h>


/* Constants */
#define BUFFER_SIZE		1024	/* Arbitrary again.  Deal. */
#define POSTAL_ERRSTR_LEN	1024	/* Ditto */
/* This may need to be redefined sometime */
#define POSTAL_ERRFOO_HASSTR (1 << 15)


/* Typedef */
typedef struct postal_errfoo_t POSTAL_ERRFOO_T;


/* Macros */
#define CLEARSTRING(s)		(bzero(s, strlen(s)))
#define UMAX(a,b)		((a) > (b) ? (a) : (b))


/*
 * Some error-checking macros.
 * We've got a number of possible cases; we can either returns ints (-1),
 * or pointers (NULL).  Also, we want debugging ability for while we're
 * developing the library, so we should add variants that describe
 * exactly where we're returning what.
 */

/* First, return() wrappers */
#ifdef POSTAL_DEBUG
 /* Debug versions */
# ifdef POSTAL_SHOW_NOERR
#  define POSTAL_SPIT_ERRINFO \
				fprintf(stderr, "Returning error %d(%s) at %s:%d\n", \
						postal_errno(), \
						postal_strerror(postal_errno()), \
						__FILE__, __LINE__);
# else  /* !POSTAL_SHOW_NOERR */
#  define POSTAL_SPIT_ERRINFO \
				if(postal_errno()!=POSTAL_E_NOERR) \
					fprintf(stderr, "Returning error %d(%s) at %s:%d\n", \
							postal_errno(), \
							postal_strerror(postal_errno()), \
							__FILE__, __LINE__);
# endif

# define POSTAL_ERRRET_RI \
			{ \
				POSTAL_SPIT_ERRINFO \
				return(-1); \
			}
# define POSTAL_ERRRET_RP \
			{ \
				POSTAL_SPIT_ERRINFO \
				return(NULL); \
			}
#else
 /* Non-debug versions */
# define POSTAL_ERRRET_RI return(-1);
# define POSTAL_ERRRET_RP return(NULL);
#endif

/* Now, setting error codes and returning */
/* For returning ints */
#define POSTAL_ERRSET_RI(num,str) \
			{ \
				postal_err_set(num,str); \
				POSTAL_ERRRET_RI; \
			}
/* For returning pointers */
#define POSTAL_ERRSET_RP(num,str) \
			{ \
				postal_err_set(num,str); \
				POSTAL_ERRRET_RP; \
			}


/* Prototypes */
/* Private interfaces */
POSTAL_CONN *postal_alloc_conn(void);
POSTAL_MSG *postal_alloc_msg(void);
char *postal_casestr(char *, char *);
int postal_dotlock(char *, int);
int postal_fcntl_lock(int, int);
int postal_flock(int, int);
int postal_flush_line(int);
void postal_free_charpp(char **);
void postal_free_conn(POSTAL_CONN *);
void postal_free_msg(POSTAL_MSG *);
int postal_get_line(int, char *, unsigned);
char **postal_list_dir(const char *);
char *postal_whack_cr(char *);
void postal_err_set(int, const char *);


/* Structures */
struct postal_errfoo_t
{
	unsigned int	postal_errno;
	char			postal_errstr[POSTAL_ERRSTR_LEN];
};

#endif /* !_LIBPOSTAL_POST_RAW_P_H_ */
