/*
 * maildir_create.c
 * Create a Maildir/
 * libpostal project
 * libpost_raw maildir code
 * $Id: maildir_create.c,v 1.4 2003/07/16 10:38:52 fullermd Exp $
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_maildir_p.h>
#include <post_raw_extra_p.h>


/* Macro for testing and making a dir */
#define TESTANDMAKE(dir)	\
		{ \
			if( stat(dir, &st_ret) == 0 ) \
			{ \
				/* Already exists */ \
				if( !(st_ret.st_mode & S_IFDIR) ) \
				{ \
					/* Not a dir.  Bail. */ \
					free(buf); \
					free(buf2); \
					POSTAL_ERRSET_RI(POSTAL_E_INVAL, NULL); \
				} \
			} \
			else \
			{ \
				/* Doesn't exist.  Bring it on, mode 700 */ \
				if( mkdir(dir, S_IRWXU) < 0 ) \
				{ \
					/* Failed */ \
					free(buf); \
					free(buf2); \
					POSTAL_ERRSET_RI(POSTAL_E_FSOP, NULL); \
				} \
			} \
		}

int
maildir_create(const char *where)
{
	struct stat st_ret;
	char *buf, *buf2;

	/*
	 * We could do a bunch of testing to see if we can do this, but why
	 * bother when we can just try and bail if we fail?
	 */

	buf = strdup(where);
	if(buf==NULL)
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	while( (strlen(buf)>0) && buf[(strlen(buf)-1)]=='/')
		buf[(strlen(buf)-1)]='\0';
	/* + '/' + '(new|cur|tmp)' + '\0' */
	buf2 = malloc( strlen(buf)+1+3+1 );
	if(buf2==NULL)
	{
		free(buf);
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	}

	/*
	 * We DO need to check that *where is either already a directory, or
	 * doesn't exist.
	 */
	TESTANDMAKE(buf);

	sprintf(buf2, "%s/new", buf);
	TESTANDMAKE(buf2);

	sprintf(buf2, "%s/cur", buf);
	TESTANDMAKE(buf2);

	sprintf(buf2, "%s/tmp", buf);
	TESTANDMAKE(buf2);

	/* If we get here, it all worked */
	free(buf2);
	free(buf);

	return(0);
}
