/*
 * maildir_get_message_all.c
 * Get all messages out of a Maildir/
 * libpostal project
 * libpost_raw maildir code
 * $Id: maildir_get_message_all.c,v 1.16 2003/07/16 11:17:30 fullermd Exp $
 */

#include <sys/param.h>
#include <stdlib.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_maildir_p.h>
#include <post_raw_extra_p.h>


long
maildir_get_message_all(POSTAL_CONN *p_conn, int get_type)
{
	POSTAL_MSG_GROUP *msgs;
	POSTAL_MSG *msgtmp;
	char dirname[PATH_MAX+1];
	int loc;
	size_t baselen;
	char **dirlist, **dirlist2, **dltmp;
	long nfiles, i;
	short onsecond; /* Just a flag */

	/*
	 * First, we're always going to put together the list of filenames
	 * themselves, which is our equivalent of GET_OFFSET.
	 */
	baselen = strlen(p_conn->spec.mdir->base);
	sprintf(dirname, "%s/cur", p_conn->spec.mdir->base);
	dirlist = postal_list_dir(dirname);
	if(dirlist==NULL)
		if(postal_errno()!=POSTAL_E_NOERR)
			POSTAL_ERRRET_RI; /* Preserve p_errno */
	sprintf(dirname, "%s/new", p_conn->spec.mdir->base);
	dirlist2 = postal_list_dir(dirname);
	if(dirlist2==NULL)
		if(postal_errno()!=POSTAL_E_NOERR)
		{
			postal_free_charpp(dirlist);
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}

	/* Patch up */
	if(dirlist==NULL)
	{
		dirlist = malloc(sizeof(char *));
		if(dirlist==NULL)
		{
			postal_free_charpp(dirlist2);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}
		dirlist[0]=NULL;
	}
	if(dirlist2==NULL)
	{
		dirlist2 = malloc(sizeof(char *));
		if(dirlist2==NULL)
		{
			postal_free_charpp(dirlist);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}
		dirlist2[0]=NULL;
	}

	/* There has to be a better way to do this */
	nfiles=0;
	for(i=0 ; dirlist[i]!=NULL ; i++)
		nfiles++;
	for(i=0 ; dirlist2[i]!=NULL ; i++)
		nfiles++;

	/* Now we know how many files to allocate, so get 'em going */
	msgs = postal_alloc_msggroup();
	if(msgs==NULL)
	{
		postal_free_charpp(dirlist);
		postal_free_charpp(dirlist2);
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	}

	msgs->msgs=malloc( (sizeof(POSTAL_MSG *) * nfiles) );
	if(msgs->msgs==NULL)
	{
		postal_free_charpp(dirlist);
		postal_free_charpp(dirlist2);
		postal_free_msggroup(msgs);
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	}
	msgs->alloced=nfiles;

	nfiles=0; /* Reuse, recycle! */
	dltmp = dirlist;
	i=0;
	onsecond=0;
	loc = LOC_CUR;
	while(dltmp[i]!=NULL)
	{
		/*
		 * XXX
		 * This has potential memory leaks of msgs and
		 * msgs->msgs[already_done].  I don't feel like thinking it
		 * through right now, so deal.
		 */
		msgtmp = postal_alloc_msg();
		if(msgtmp==NULL)
		{
			postal_free_charpp(dirlist);
			postal_free_charpp(dirlist2);
			postal_free_msggroup(msgs);
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}

		msgtmp->src_type = SRC_MDIR;
		/* XXX Make a alloc function */
		msgtmp->loc.file = maildir_alloc_mdmsg();
		if(msgtmp->loc.file==NULL)
		{
			postal_free_charpp(dirlist);
			postal_free_charpp(dirlist2);
			postal_free_msg(msgtmp);
			postal_free_msggroup(msgs);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}

		msgtmp->loc.file->loc = loc;
		msgtmp->loc.file->obase = strdup(p_conn->spec.mdir->base);
		if(msgtmp->loc.file->obase==NULL)
		{
			postal_free_charpp(dirlist);
			postal_free_charpp(dirlist2);
			postal_free_msg(msgtmp);
			postal_free_msggroup(msgs);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}

		/* Smack in the filename */
		msgtmp->loc.file->filename = strdup(dltmp[i]);
		if(msgtmp->loc.file->filename==NULL)
		{
			postal_free_charpp(dirlist);
			postal_free_charpp(dirlist2);
			postal_free_msg(msgtmp);
			postal_free_msggroup(msgs);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}

		/* Derive info from the filename */
		if(maildir_msg_filename_dissect(msgtmp->loc.file)<0)
		{
			postal_free_charpp(dirlist);
			postal_free_charpp(dirlist2);
			postal_free_msg(msgtmp);
			postal_free_msggroup(msgs);
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}

		/* Cheat a little for the GET_OFFSET case */
		if(get_type!=GET_OFFSET)
			if(maildir_get_message(p_conn, msgtmp, get_type)<0)
			{
				postal_free_charpp(dirlist);
				postal_free_charpp(dirlist2);
				postal_free_msg(msgtmp);
				postal_free_msggroup(msgs);
				POSTAL_ERRRET_RI; /* Preserve p_errno */
			}
				

		/* Stuff it in */
		(msgs->msgs)[nfiles] = msgtmp;
		msgs->num_msgs++;
		nfiles++;
		i++;

		/* Hop on to the second dir listing */
		if( (dltmp[i]==NULL) && (onsecond==0) )
		{
			i=0;
			dltmp = dirlist2;
			onsecond=1;
			loc = LOC_NEW;
		}
	}

	/* So, we should be all populated and ready to run now */
	p_conn->msgs = msgs;
	postal_free_charpp(dirlist);
	postal_free_charpp(dirlist2);
	return(msgs->num_msgs);
}
