/*
 * maildir_msg_filename_dissect.c
 * Split a Maildir message filename into its parts
 * libpostal project
 * libpost_raw maildir code
 * $Id: maildir_msg_filename_dissect.c,v 1.4 2003/06/04 05:55:05 fullermd Exp $
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_maildir_p.h>
#include <post_raw_extra_p.h>


int
maildir_msg_filename_dissect(MAILDIR_MSG *minfo)
{
	char *tmpnam, *tmpntmp, *atsize, *atinfo, *buf;

	/* Our location can effect what we get out */
	if(minfo->loc == LOC_NEW)
	{
		/* In new/, there's not much to do */
		if(minfo->unique!=NULL)
			free(minfo->unique);
		minfo->unique = strdup(minfo->filename);
		if(minfo->unique==NULL)
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	}
	else if(minfo->loc == LOC_CUR)
	{
		/* If we're in cur/, we've got more to parse */
		tmpnam = strdup(minfo->filename);
		if(tmpnam==NULL)
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);

		/* Find the 'info' segment and save it */
		atinfo = strchr(tmpnam, ':');
		*atinfo = '\0';
		atinfo++;
		if(minfo->info!=NULL)
			free(minfo->info);
		minfo->info = strdup(atinfo);
		if(minfo->info==NULL)
		{
			free(tmpnam);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}

		/* Tricky!  No seatbelts! */
		tmpntmp = realloc(tmpnam, strlen(tmpnam)+1);
		if(tmpntmp==NULL)
		{
			free(tmpnam);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}
		tmpnam = tmpntmp;

		/* See if we have a Courier-style size specified */
		atsize = strstr(tmpnam, ",S=");
		if(atsize!=NULL)
		{
			/* We do.  Translate and save it */
			*atsize = '\0';
			atsize += 3;
			buf = atsize;

			/* We shouldn't have anything after this */
			while(isdigit((int)*buf))
				buf++;
			if(*buf!='\0')
			{
				/*
				 * XXX 
				 * There's other stuff after the ,S=xxx.
				 * We don't currently handle that.  We should.
				 * WRITEME 
				 * XXX 
				 */ 
			}
			else
			{
				/* We have a number, so translate it. */
				/*
				 * XXX
				 * off_t is often a 64-bit type.  We don't have an
				 * equivalent to atol() for quad or long long in C89,
				 * only in C99, and it's a bit early to expect that to be
				 * ubiquitous.  So, for now, you'll have to just hope
				 * that, on a 32-bit machine, you don't have a email
				 * message bigger than 2 gig.
				 */ 
				minfo->size = (off_t) atol(atsize);
			}
		}

		/*
		 * What's left is the 'unique' part we don't attempt to
		 * subdivide.
		 */ 
		if(minfo->unique!=NULL)
			free(minfo->unique);
		minfo->unique= strdup(tmpnam);
		if(minfo->unique==NULL)
		{
			free(tmpnam);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}

		/* Clean up.  We're outta this cheesebarrel */
		free(tmpnam);
	}
	else
	{
		/* We don't yet cover any other cases, and maybe never will */
		POSTAL_ERRSET_RI(POSTAL_E_WHOKNOWS, NULL);
	}

	return(0);
}
