/*
 * maildir_open.c
 * Open a Maildir/
 * libpostal project
 * libpost_raw maildir code
 * $Id: maildir_open.c,v 1.7 2003/07/16 09:44:14 fullermd Exp $
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_maildir_p.h>
#include <post_raw_extra_p.h>


/* Macro for testing dir-ness */
#define TESTDIR(dir) \
		{ \
			if( stat(dir, &st_ret) == 0 ) \
			{ \
				/* It exists */ \
				if( !(st_ret.st_mode & S_IFDIR) ) \
				{ \
					/* Not a dir, bail */ \
					free(buf); \
					free(buf2); \
					postal_free_conn(toret); \
					POSTAL_ERRSET_RP(POSTAL_E_INVAL, NULL); \
				} \
			} \
			else \
			{ \
				/* \
				 * Doesn't exist.  Create if we're told to, else bail and \
				 * run for it. \
				 */ \
				if(create==1) \
				{ \
					if( maildir_create(loc) < 0 ) \
					{ \
						free(buf); \
						free(buf2); \
						postal_free_conn(toret); \
						POSTAL_ERRRET_RP; /* Preserve p_errno */ \
					} \
					/* Shortcut */ \
					goto end; \
				} \
				else \
				{ \
					free(buf); \
					free(buf2); \
					postal_free_conn(toret); \
					POSTAL_ERRSET_RP(POSTAL_E_NEXIST, NULL); \
				} \
			} \
		}



POSTAL_CONN *
maildir_open(const char *loc, int create)
{
	POSTAL_CONN *toret;
	struct stat st_ret;
	char *buf, *buf2;

	/* Allocate memory */
	toret = postal_alloc_conn();
	if(toret==NULL)
		POSTAL_ERRRET_RP; /* Preserve p_errno */
	toret->spec.mdir = maildir_alloc_spec();
	if(toret->spec.mdir==NULL)
	{
		postal_free_conn(toret);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	toret->conn_type = SRC_MDIR;

	/* Copy the location into the SPEC_ */
	toret->spec.mdir->base = malloc( (strlen(loc)+1) );
	if(toret->spec.mdir->base==NULL)
	{
		postal_free_conn(toret);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	strcpy(toret->spec.mdir->base, loc);

	/* Double- and triple-check if it's really a Maildir/ */
	buf = strdup(loc);
	if(buf==NULL)
	{
		postal_free_conn(toret);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	while( (strlen(buf)>0) && buf[(strlen(buf)-1)]=='/')
		buf[(strlen(buf)-1)]='\0';
	/* + '/' + '(new|cur|tmp)' + '\0' */
	buf2 = malloc( strlen(buf)+1+3+1 );
	if(buf2==NULL)
	{
		free(buf);
		postal_free_conn(toret);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}

	TESTDIR(buf);

	sprintf(buf2, "%s/new", buf);
	TESTDIR(buf2)

	sprintf(buf2, "%s/cur", buf);
	TESTDIR(buf2)

	sprintf(buf2, "%s/tmp", buf);
	TESTDIR(buf2)

end:
	/* If we make it to here, we're ready to head onward */
	free(buf);
	free(buf2);
	return(toret);
}
