/*
 * mbox_get_message_all.c
 * Get all messages from a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_get_message_all.c,v 1.41 2003/07/16 10:12:40 fullermd Exp $
 */

#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>

/* # of message slots to allocate at once */
#define	ALLOC_CHUNK		128
/* 16k.  Size to read() at once.  Tune this */
#define READ_CHUNK	16384
/* Number of chunks to malloc() at a time.  16x16=256 */
#define	MALLOC_NUM	16
/* Size limits.  How many chunks to limit at. 8x16x16=2048 (2 megs) */
#define	MAX_CHUNKS 8

/* We're going to abuse macros a lot */
/* Do a read() cycle */
#define READ_A_CHUNK \
						{ \
							VERIFY_SIZES; \
							VERIFY_AVAIL; \
							if( (readlen = read(p_conn->spec.mbox->desc, \
									rawloc, (bufavail-1))) \
									> 0) \
							{ \
								rawloc[readlen] = '\0'; \
								rawloc += readlen; \
								bufavail -= readlen; \
								readsz += readlen; \
							} \
							else if(readlen==0) \
							{ \
								/* EOF */ \
								done=1; \
								goto endofwhile;  /* Goto's rule! */ \
							} \
							else \
							{ \
								/* Error */ \
								free(rawbuf); \
								postal_free_msggroup(toret); \
								postal_free_msg(p_msg); \
								POSTAL_ERRSET_RI(POSTAL_E_FSOP, NULL); \
							} \
						}
/* End of READ_A_CHUNK */

/* Alloc a chunk for read() */
#define ALLOC_READ_CHUNK \
						{ \
							if( (bufsz+READ_CHUNK) > allocsz ) \
							{ \
								long i = startat-rawbuf; \
								long j = rawloc-startat; \
								rawbtmp = realloc(rawbuf, \
										(allocsz+(READ_CHUNK*MALLOC_NUM))); \
								if(rawbtmp==NULL) \
								{ \
									free(rawbuf); \
									postal_free_msggroup(toret); \
									postal_free_msg(p_msg); \
									POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL); \
								} \
								rawbuf = rawbtmp; \
								startat = rawbuf+i; \
								rawloc = startat+j; \
								allocsz += (READ_CHUNK*MALLOC_NUM); \
							} \
							bufsz += READ_CHUNK; \
							bufavail += READ_CHUNK; \
							rawloc = startat + readsz; \
						}
/* End of ALLOC_READ_CHUNK */

#ifdef LOTS_OF_DEBUGGING
#define SHOWLOC		printf("At %s, line %d: ", __FILE__, __LINE__)

#define VERIFY_AVAIL \
				{ \
					if( (rawloc + bufavail - rawbuf) > allocsz ) \
					{ \
						SHOWLOC; \
						printf("About to go too far (alloc)\n"); \
						printf("rawloc(%li)+bufavail(%li)-rawbuf(%li)" \
							"(=%li) > allocsz(%li)\n", (long)rawloc, \
							(long)bufavail, (long)rawbuf, \
							(long)(rawloc+bufavail-rawbuf), \
							(long)allocsz); \
						exit(1); \
					} \
					if( (rawloc + bufavail - rawbuf) > bufsz ) \
					{ \
						SHOWLOC; \
						printf("About to go too far (logical alloc)\n"); \
						printf("rawloc(%li)+bufavail(%li)-rawbuf(%li)" \
							"(=%li) > bufsz(%li)\n", (long)rawloc, \
							(long)bufavail, (long)rawbuf, \
							(long)(rawloc+bufavail-rawbuf), \
							(long)bufsz); \
						exit(1); \
					} \
				}

#define VERIFY_SIZES \
				{ \
					if( (rawloc < startat) ) \
					{ \
						SHOWLOC; \
						printf("Writing ahead of start\n"); \
						printf("rawloc(%li) < startat(%li)\n", \
							(long)rawloc, (long)startat); \
						exit(1); \
					} \
					if( (readsz+bufavail) > allocsz ) \
					{ \
						SHOWLOC; \
						printf("Too much space used\n"); \
						printf("readsz(%li)+bufavail(%li)(=%li)" \
							" > allocsz(%li)\n", (long)readsz, \
							(long)bufavail, \
							(long)(readsz+bufavail), \
							(long)allocsz); \
						exit(1); \
					} \
					if( (startat + readsz) > (rawbuf + allocsz) ) \
					{ \
						SHOWLOC; \
						printf("Gone too far (pure alloc)\n"); \
						printf("startat(%li)+readsz(%li)(=%li)" \
							" > rawbuf(%li)+allocsz(%li)(=%li)", \
							(long)startat,(long)readsz,(long)startat+readsz, \
							(long)rawbuf,(long)allocsz,(long)rawbuf+allocsz); \
						exit(1); \
					} \
					if( (startat + readsz) > (rawbuf + bufsz) ) \
					{ \
						SHOWLOC; \
						printf("Gone too far (logical alloc)\n"); \
						printf("startat(%li)+readsz(%li)(=%li)" \
							" > rawbuf(%li)+bufsz(%li)(=%li)", \
							(long)startat,(long)readsz,(long)startat+readsz, \
							(long)rawbuf,(long)bufsz,(long)rawbuf+bufsz); \
						exit(1); \
					} \
				}
#else /* NOT lots of debugging */
#define VERIFY_AVAIL	 (void)0
#define VERIFY_SIZES	(void)0
#endif



long
mbox_get_message_all(POSTAL_CONN *p_conn, int get_type)
{
	POSTAL_MSG_GROUP *toret;
	POSTAL_MSG *p_msg, **msgstmp;
	char *rawbuf=NULL, *rawbtmp, *rawloc, *startat;
	char *buf, *buf2;
	char *hdrs, *msg;
	size_t bufavail, bufsz, allocsz, hdrlen, bodylen, morelen, msglen;
	ssize_t readlen, readsz;
	long conlen;
	int done=0;
	int notfrom;
	off_t offset;

	toret = postal_alloc_msggroup();
	if(toret==NULL)
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);

	/* We need to start at the beginning of the file of course */
	lseek(p_conn->spec.mbox->desc, 0, SEEK_SET);
	offset=0;
	readsz=0;

	/*
	 * We COULD just keep calling mbox_get_message_next(), but that's far
	 * from the most efficient way to do this for getting ALL the
	 * messages.
	 * Parse it all manually.
	 */
	rawbuf = malloc(READ_CHUNK*MALLOC_NUM);
	if(rawbuf==NULL)
	{
		postal_free_msggroup(toret);
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	}
	rawloc = rawbuf;
	startat = rawbuf;
	allocsz = READ_CHUNK*MALLOC_NUM;
	bufsz = READ_CHUNK;
	bufavail = READ_CHUNK;
	VERIFY_SIZES;
	
	VERIFY_AVAIL;
	if( (readlen = read(p_conn->spec.mbox->desc, rawloc, (bufavail-1)))
			> 0)
	{
		rawloc[readlen] = '\0';
		rawloc += readlen;
		bufavail -= readlen;
		readsz += readlen;
	}
	else
	{
		/*
		 * Either EOF, or an error.
		 * If we EOF here, there's NO messages, which is an error.
		 * Bail.
		 */
		free(rawbuf);
		postal_free_msggroup(toret);
		postal_free_msg(p_msg);
		if(readlen==0)
			POSTAL_ERRSET_RI(POSTAL_E_DFORM, NULL)
		else
			POSTAL_ERRSET_RI(POSTAL_E_FSOP, NULL);
	}

	/* We'll go through this loop once per message */
	while(!done)
	{
topwhile:
		VERIFY_SIZES;
		msglen=-1; /* This is probably unsigned, so this is fun */
		conlen=0;
		p_msg = postal_alloc_msg();
		if(p_msg==NULL)
		{
			free(rawbuf);
			postal_free_msggroup(toret);
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}
		p_msg->loc.offset = offset;
		p_msg->src_type = SRC_MBOX;

		/* First, seperate out the headers */
		while( !(buf=strstr(startat, "\n\n")) )
		{
			/*
			 * What, we have no HEADERS?  With the READ_CHUNK's we're
			 * using?!
			 * Jeez.  Well, keep trying...
			 */
			ALLOC_READ_CHUNK;
			READ_A_CHUNK;
		}

		/*
		 * We have a full header.
		 * Include the \n at the end, but not the blank line after it.
		 */
		buf++;
		hdrlen = buf-startat;
		hdrs = malloc(hdrlen+1);
		if(hdrs==NULL)
		{
			free(rawbuf);
			postal_free_msg(p_msg);
			postal_free_msggroup(toret);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}
		strncpy(hdrs, startat, hdrlen);
		hdrs[hdrlen]='\0'; /* Stupid strncpy... */

		/* Check for Content-Length: */
		VERIFY_SIZES;
		buf=postal_get_header(hdrs, "Content-Length:");
		if( (buf==NULL) && (postal_errno()!=POSTAL_E_NOERR) )
		{
			free(hdrs);
			free(rawbuf);
			postal_free_msg(p_msg);
			postal_free_msggroup(toret);
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}

		if(buf!=NULL)
		{
			conlen = atol(buf);
			free(buf);
			VERIFY_SIZES;

			/*
			 * We have the length parsed out.
			 * See comments in mbox_get_message() for details on how some
			 * of this stuff fits together, I don't feel like retyping it
			 * all.
			 */
			if( readsz < (hdrlen+conlen+1) )
			{
				/*
				 * We don't have the whole message read() in.
				 * read() the rest, since we know just how big it is.
				 * Add a bit extra for optimization purposes, since
				 * 1 32k read is a lot better than 2 16k reads.
				 */
				morelen = (hdrlen + conlen + 1) - readsz + READ_CHUNK;
				if( (bufsz+morelen+1) > allocsz )
				{
					buf2 = malloc(allocsz+morelen+1);
					if(buf2==NULL)
					{
						free(hdrs);
						free(rawbuf);
						postal_free_msg(p_msg);
						postal_free_msggroup(toret);
						POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
					}
					strncpy(buf2, startat, readsz);
					buf2[readsz]='\0';
					free(rawbuf);
					startat = rawbuf = buf2;
					allocsz += morelen + 1;
					rawloc = startat + readsz;
					VERIFY_SIZES;
				}
				VERIFY_SIZES;
				bufsz += morelen+1;
				bufavail += morelen+1;
				VERIFY_SIZES;
				VERIFY_AVAIL;
				readlen = read(p_conn->spec.mbox->desc, rawloc,
						(bufavail-1));
				if(readlen<=0)
				{
					free(hdrs);
					free(rawbuf);
					postal_free_msg(p_msg);
					postal_free_msggroup(toret);
					if(readlen==0)
						POSTAL_ERRSET_RI(POSTAL_E_DFORM, NULL)
					else
						POSTAL_ERRSET_RI(POSTAL_E_FSOP, NULL);
				}
				rawloc[readlen]='\0';
				rawloc += readlen;
				bufavail -= readlen;
				readsz += readlen;
				VERIFY_SIZES;
				/* See mbox_get_message() notes about sanity checks */
				msglen = hdrlen + conlen + 1;
				/* FALLTHRU */
			}
			else
			{
				/* Already got the whole thing */
				msglen = hdrlen + conlen + 1;
				/* FALLTHRU */
			}
		} /* Have Content-Length:? */
		else
		{
			/*
			 * Have to do things the old-fashioned (read: fragile) way,
			 * and read ahead until we find a From_ line.
			 */
			while( !(buf=strstr(startat, "\n\nFrom ")) )
			{
				/* Need to read more.  MORE, dammit! */
				ALLOC_READ_CHUNK;
				READ_A_CHUNK;
			}

			/* We've got one */
			buf+=2;

fromtest:
			/* Check for From_-ness */
			notfrom=0;
			/*
			 * XXX Special case hack.
			 * From_ 'should' be, say, at least 80 chars.
			 */
			if(strlen(buf)<80)
			{
				morelen=buf-startat;
				ALLOC_READ_CHUNK;
				READ_A_CHUNK;
				buf=startat+morelen;
				goto fromtest;
			}
			notfrom=mbox_validate_from(buf);

			if(notfrom==0)
			{
				/* Got a full message */
				msglen = (buf-1)-startat;
				/* FALLTHRU */
			}
			else
			{
				/* It's not a From_ really.  Fake some stuff */
				buf2 = buf;
				while( !(buf=strstr(buf2, "\n\nFrom ")) )
				{
					ALLOC_READ_CHUNK;
					READ_A_CHUNK;
				}
				goto fromtest; /* Purrrty, ainit? */
			}
		} /* Search for "^From " */

endofwhile:
		/*
		 * This is where things goto to when we read EOF, as well as
		 * where we fall-through to.
		 * If we get here, there's a full message sitting around, and we
		 * know how long it is.
		 * Parse it out into the struct here, and head back around.
		 */

		/* If msglen=-1, we hit EOF somewhere */
		if(msglen==-1)
		{
			msglen = strlen(startat)-1;
			done=1;
		}

		startat[msglen]='\0'; /* Kill \n at the end */

		/* Seperate header and body, pack the struct */
		if( (get_type==GET_HEADER) || (get_type == GET_BODY) )
		{
			p_msg->header = hdrs;
			hdrs=0;
			p_msg->hdrlen = hdrlen;
			p_msg->stat_loc = mbox_get_status(p_msg);
			if(p_msg->stat_loc==-1)
			{
				free(rawbuf);
				postal_free_msg(p_msg);
				postal_free_msggroup(toret);
				POSTAL_ERRRET_RI; /* Preserve p_errno */
			}
			if(get_type==GET_BODY)
			{
				buf=startat+hdrlen+1;
				if(hdrlen+1>msglen)
					bodylen=0;
				else
					bodylen = msglen - (hdrlen+1);
				msg = malloc(bodylen+1);
				if(msg==NULL)
				{
					free(rawbuf);
					postal_free_msg(p_msg);
					postal_free_msggroup(toret);
					POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
				}
				strncpy(msg, buf, bodylen);
				msg[bodylen]='\0'; /* Stupid strncpy... */
				p_msg->body = msg;
				p_msg->bodylen = bodylen;
				msg=NULL;
			}
		}
		else
			free(hdrs);

		/* Now frob some stuff */
		buf = startat + msglen + 1;
		if(*buf && strncmp(buf, "From ", 5))
		{
			/* Not right */
			exit(1);
		}
		offset = p_msg->loc.offset + msglen + 1;
		VERIFY_SIZES;
		morelen = readsz-msglen-1;
		readsz = morelen;
		VERIFY_SIZES;
		if( !done &&
				((morelen < 256)
				|| ((bufsz+READ_CHUNK) >= allocsz)
				|| allocsz >= (MAX_CHUNKS*MALLOC_NUM*READ_CHUNK)) )
		{
			/*
			 * We are:
			 *  - Almost out of chars, so strncpy() is very little
			 *    overhead, or
			 *  - Going to run out of space in our allocation vewy vewy
			 *    soon, so it's no big loss, or
			 *  - Over our specified max size, so we better cut down
			 * We're going to need to realloc() very soon, or we have
			 * almost nothing to copy over, so this should be either
			 * necessary or efficient.
			 */
			buf2 = malloc(morelen+MALLOC_NUM*READ_CHUNK);
			if(buf2==NULL)
			{
				free(rawbuf);
				postal_free_msg(p_msg);
				postal_free_msggroup(toret);
				POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
			}
			strncpy(buf2, buf, morelen);
			buf2[morelen]='\0';
			free(rawbuf);
			startat = rawbuf = buf2;
			allocsz = morelen + MALLOC_NUM*READ_CHUNK;
			bufsz = morelen + READ_CHUNK;
			bufavail = READ_CHUNK;
			rawloc = rawbuf + morelen;
			VERIFY_SIZES;
		}
		else
		{
			startat = buf;
			VERIFY_SIZES;
		}

		/* Add another item into our big return structure */
		/*
		 * XXX Should we move this above the block above the
		 * manual-realloc() block above, and adjust the free()'ing in the
		 * error return to match?
		 */	
		toret->num_msgs++;
		if( toret->alloced < (toret->num_msgs) )
		{
			/* Need more space */
			msgstmp = realloc( toret->msgs,
					(sizeof(POSTAL_MSG *) * (toret->alloced + ALLOC_CHUNK)) );
			if(msgstmp==NULL)
			{
				free(rawbuf);
				postal_free_msg(p_msg);
				postal_free_msggroup(toret);
				POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
			}
			toret->msgs = msgstmp;
			toret->alloced += ALLOC_CHUNK;
		}
		(toret->msgs)[(toret->num_msgs-1)] = p_msg;

		/* XXX Hardcoded sanity size */
		if(!done && morelen < 2048)
		{
			VERIFY_SIZES;
			/*
			 * read() another block to prepare to loop back around.
			 * This is copied from before the while().  We should macro it.
			 */
			VERIFY_AVAIL;
			if( (readlen = read(p_conn->spec.mbox->desc, rawloc, (bufavail-1)))
					> 0)
			{
				rawloc[readlen] = '\0';
				rawloc += readlen;
				bufavail -= readlen;
				readsz += readlen;
				VERIFY_SIZES;
			}
			else if( (readlen==0) && (morelen>0) )
			{
				/* EOF, but more messages already buffered */
				goto topwhile;
			}
			else if(readlen==0)
			{
				/* We're ready to return */
				done=1;
			}
			else
			{
				/* Bail */
				free(rawbuf);
				postal_free_msggroup(toret);
				postal_free_msg(p_msg);
				POSTAL_ERRSET_RI(POSTAL_E_FSOP, NULL);
			}
		}
		VERIFY_SIZES;
	} /* while(!done) */

	/* Prune our memory a bit */
	free(rawbuf);
	msgstmp = realloc (toret->msgs,
			(sizeof(POSTAL_MSG *) * (toret->num_msgs)) );
	if(msgstmp==NULL)
	{
		postal_free_msggroup(toret);
		POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
	}
	toret->msgs = msgstmp;
	toret->alloced = toret->num_msgs;
	
	p_conn->msgs = toret;
	return(toret->num_msgs);
}
