/*
 * mbox_get_status.c
 * Get status info from a mbox-derived message
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_get_status.c,v 1.8 2003/06/05 11:19:30 fullermd Exp $
 */

#include <stdlib.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>


int
mbox_get_status(POSTAL_MSG *msg)
{
	char *hdrs;
	int statae=STATUS_NULL;

	if(msg->hdrlen==0)
		POSTAL_ERRSET_RI(POSTAL_E_INVAL, NULL);

	/* Find the Status: header and copy it down locally */
	hdrs = postal_get_header(msg->header, "Status:");
	if( (hdrs==NULL) && (postal_errno()!=POSTAL_E_NOERR) )
		POSTAL_ERRRET_RI; /* Preserve p_errno */
	if(hdrs)
	{
		/* 'O' means 'old' */
		if(strchr(hdrs, 'O'))
			statae |= STATUS_OLD;
		else
			statae |= STATUS_NEW;
	
		/* 'R' means 'read' */
		if(strchr(hdrs, 'R'))
			statae |= STATUS_READ;

		free(hdrs);
	}
	else
		statae |= STATUS_NEW;

	/*
	 * Try X-Status: too, a number of mailers (mutt, pine, etc) use it
	 * too.
	 */
	hdrs = postal_get_header(msg->header, "X-Status:");
	if( (hdrs==NULL) && (postal_errno()!=POSTAL_E_NOERR) )
		POSTAL_ERRRET_RI; /* Preserve p_errno */
	if(hdrs)
	{
		/* 'A' means 'replied to' */
		if(strchr(hdrs, 'A'))
			statae |= STATUS_REPLIED;

		/* There are some others, but I don't think they really matter */

		free(hdrs);
	}

	return(statae);
}
