/*
 * mbox_lock_read.c
 * Lock an mbox for reading ('shared')
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_lock_read.c,v 1.8 2003/06/05 11:19:30 fullermd Exp $
 */

#include <sys/file.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>


int
mbox_lock_read(POSTAL_CONN *mbox, int locks)
{
	/*
	 * Check the lock types and execute.
	 * This is our canonical order.
	 */

	/* flock() */
	if( (locks & LOCK_FLOCK) &&
		( !(mbox->spec.mbox->locks & LOCK_FLOCK)
			|| (mbox->spec.mbox->locks & LOCK_FLOCK_EX) ) )
	{
		if( (postal_flock(mbox->spec.mbox->desc, LOCK_SH)!=0) )
		{
			/* Something went wrong */
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}
		/* Note that we're flock()-locked */
		mbox->spec.mbox->locks |= LOCK_FLOCK;
		mbox->spec.mbox->locks &= ~(LOCK_FLOCK_EX);
	}

	/* dotlock */
	if((locks & LOCK_DOTLOCK) && !(mbox->spec.mbox->locks & LOCK_DOTLOCK))
	{
		if( (postal_dotlock(mbox->spec.mbox->filename, LOCK_SH)!=0) )
		{
			/* Something went wrong */
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}
		/* Note that we're dotlocked */
		mbox->spec.mbox->locks |= LOCK_DOTLOCK;
	}

	/* fcntl */
	if((locks & LOCK_FCNTL) && !(mbox->spec.mbox->locks & LOCK_FCNTL))
	{
		if( (postal_fcntl_lock(mbox->spec.mbox->desc, LOCK_SH)!=0) )
		{
			/* Something went wrong */
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}
		/* Note that we're fcntl-locked */
		mbox->spec.mbox->locks |= LOCK_FCNTL;
	}

	/* No other locktypes defined yet */

	/* If we got here, everything succeeded */
	return(0);
}
