/*
 * mbox_open_file.c
 * Open a file to use as a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_open_file.c,v 1.23 2003/07/14 21:07:34 fullermd Exp $
 */

#include <sys/file.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>


POSTAL_CONN *
mbox_open_file(const char *fname, int locktype, int readwrite)
{
	POSTAL_CONN *toret;
	struct stat st_ret;

	/* Allocate memory */
	toret = postal_alloc_conn();
	if(toret==NULL)
		POSTAL_ERRRET_RP; /* Preserve p_errno */
	toret->spec.mbox = mbox_alloc_spec();
	if(toret->spec.mbox==NULL)
	{
		postal_free_conn(toret);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	toret->conn_type = SRC_MBOX;

	/* Copy the filename into the SPEC_ */
	toret->spec.mbox->filename = malloc( (strlen(fname)+1) );
	if(toret->spec.mbox->filename==NULL)
	{
		postal_free_conn(toret);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	strcpy(toret->spec.mbox->filename, fname);

	/*
	 * Open the file as a raw I/O stream, lock it, and get a
	 * stdio stream, since it's more fun to manipulate.
	 * Check READONLY or READWRITE
	 */
	if( (readwrite == OPEN_READONLY) )
	{
		if((toret->spec.mbox->desc = open(fname, O_RDONLY))<0)
		{
			postal_free_conn(toret);
			POSTAL_ERRSET_RP(POSTAL_E_FSOP, NULL);
		}
		toret->spec.mbox->file = fdopen(toret->spec.mbox->desc, "r");
		if(toret->spec.mbox->file==NULL)
		{
			close(toret->spec.mbox->desc);
			postal_free_conn(toret);
			POSTAL_ERRSET_RP(POSTAL_E_FSOP, NULL);
		}
		if(mbox_lock_read(toret, locktype)<0)
		{
			fclose(toret->spec.mbox->file);
			mbox_unlock(toret); /* Shouldn't have to */
			postal_free_conn(toret);
			POSTAL_ERRRET_RP; /* Preserve p_errno */
		}
	}
	else /* OPEN_READWRITE is the only current case */
	{
		if((toret->spec.mbox->desc = open(fname, O_RDWR | O_CREAT))<0)
		{
			postal_free_conn(toret);
			POSTAL_ERRSET_RP(POSTAL_E_FSOP, NULL);
		}
		toret->spec.mbox->file = fdopen(toret->spec.mbox->desc, "r+");
		if(toret->spec.mbox->file==NULL)
		{
			close(toret->spec.mbox->desc);
			postal_free_conn(toret);
			POSTAL_ERRSET_RP(POSTAL_E_FSOP, NULL);
		}
		if(mbox_lock_write(toret, locktype)!=0)
		{
			fclose(toret->spec.mbox->file);
			mbox_unlock(toret); /* Shouldn't have to */
			postal_free_conn(toret);
			POSTAL_ERRRET_RP; /* Preserve p_errno */
		}
	}

	/*
	 * Set the permissions in a portable way.
	 * XXX Should we really hardcode this?
	 */
	fchmod(toret->spec.mbox->desc, S_IRUSR | S_IWUSR);

	/* Get the stat info */
	fstat(toret->spec.mbox->desc, &st_ret);

	/* Return the structure, we're ready to rock */
	return(toret);
}
