/*
 * mbox_unlock.c
 * Unlock a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_unlock.c,v 1.9 2003/06/05 11:19:30 fullermd Exp $
 */

#include <sys/file.h>
#include <fcntl.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>


int
mbox_unlock(POSTAL_CONN *mbox)
{
	/*
	 * Check the lock types and execute.
	 * This is our canonical order.
	 */

	/* Are we flock()'d? */
	if( (mbox->spec.mbox->locks & LOCK_FLOCK) )
	{
		if( (postal_flock(mbox->spec.mbox->desc, LOCK_UN)!=0) )
		{
			/* Something went wrong */
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}
		/* Note that we're flock()-unlocked */
		mbox->spec.mbox->locks &= ~(LOCK_FLOCK | LOCK_FLOCK_EX);
	}

	/* Are we dotlocked? */
	if( (mbox->spec.mbox->locks & LOCK_DOTLOCK) )
	{
		if( (postal_dotlock(mbox->spec.mbox->filename, LOCK_UN)!=0) )
		{
			/* Something went wrong */
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}
		/* Note that we're dotlock-unlocked */
		mbox->spec.mbox->locks &= ~LOCK_DOTLOCK;
	}

	/* Are we fcntl()-locked? */
	if( (mbox->spec.mbox->locks & LOCK_FCNTL) )
	{
		if( (postal_fcntl_lock(mbox->spec.mbox->desc, LOCK_UN)!=0) )
		{
			/* Something went wrong */
			POSTAL_ERRRET_RI; /* Preserve p_errno */
		}
		/* Note that we're fcntl()-unlocked */
		mbox->spec.mbox->locks &= ~LOCK_FCNTL;
	}

	/* No other locktypes defined yet */

	/* If we got here, everything succeeded */
	return(0);
}
