/*
 * mbox_validate_from.c
 * Validate a From_ line
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_validate_from.c,v 1.5 2003/06/04 08:24:50 fullermd Exp $
 */

#include <ctype.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>


int
mbox_validate_from(const char *hdr)
{
	/*
	 * We presume *hdr is beginning with the From itself.
	 * Sanity check.
	 */
	if(strncmp(hdr, "From ", 5) != 0)
		return(-1);

	/*
	 * POLICY:
	 * There are a number of ways From_ creation could be broken.
	 * I'm not going to try to catch all of them.  Yet.
	 *
	 * This can be tricky.  We're going to use
	 * a half-way heuristic that SHOULD only trigger
	 * on real From_ lines.  mbox-files aren't
	 * REQUIRED to >-escape lines beginning with
	 * From_ unless they actually match From_-format
	 * pretty closely.  Ho-hum.  We always will, most
	 * sane programs always do, but we need to be a
	 * little careful about ones that don't.
	 */

	/* Space past From_ */
	hdr+=5;

	/* Envelope sender */
	while( ((isprint(*hdr)) && (*hdr!=' ')) )
		hdr++;

	/*
	 * Should be 1 space after.  We'll accept up to 3 for safety's sake,
	 * any more and we're sending you back to mbox-school.
	 * If we go ahead and take more than 3, probably should write this
	 * down into a real loop with a counter, but this way saves us a
	 * variable and some math in the general case.
	 * Of course, any decent compiler will spit this out ANYWAY, but
	 * that's not the point.
	 */
	hdr++;
	if(*hdr==' ')
	{
		hdr++;
		if(*hdr==' ')
		{
			hdr++;
			if(*hdr==' ')
			{
				return(-1);
			}
		}
	}

	/* Day of the week.  3 chars */
	hdr+=3;
	if(*hdr!=' ')
		return(-1);
	hdr++;

	/* Month.  3 chars */
	hdr+=3;
	if(*hdr!=' ')
		return(-1);
	hdr++;

	/* Day of the month.  If date is 1-9, there are 2 spaces */
	if(*hdr==' ')
		hdr++;
	if(!isdigit(*hdr))
		return(-1);


	/*
	 * That's all the checking I'm going to bother with for now.
	 * We may choose to add more thorough checks later, but I think this
	 * should be a great deal of plenty for now, eh?
	 */

	return(0);
}
