/*
 * mbox_write_msg.c
 * Write a message into a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_write_msg.c,v 1.16 2003/06/05 11:19:30 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>


int
mbox_write_msg(POSTAL_CONN *p_conn, POSTAL_MSG *msg, int addwhere)
{
	POSTAL_MSG *temp;
	/* XXX - Use a pointer to more easily tell if we've re-seek'd */
	fpos_t *savedpos;

	/* We need to cure \r\n's if we have them... */
	/*
	 * XXX This is a target for optimization.
	 * Waste some memory to save some CPU.  Not worrying right now.
	 */
	temp = postal_alloc_msg();
	if(temp==NULL)
		POSTAL_ERRRET_RI; /* Preserve p_errno */

	/* Compress this for my finger's sake */
	temp->body = postal_whack_cr(msg->body);
	temp->header = postal_whack_cr(msg->header);
	if( (temp->body==NULL)
			|| (temp->header==NULL)
			|| ((mbox_derive_from(&temp->header))<0)
			|| ((mbox_set_conlen(temp))<0)
			|| ((mbox_set_status(temp))<0) )
	{
		postal_free_msg(temp);
		POSTAL_ERRRET_RI; /* Preserve p_errno */
	}
	temp->bodylen = strlen(temp->body);
	temp->hdrlen = strlen(temp->header);

	/* Get to the right position */
	if(addwhere==POS_END)
	{
		savedpos = malloc(sizeof(fpos_t));
		if(savedpos==NULL)
		{
			postal_free_msg(temp);
			POSTAL_ERRSET_RI(POSTAL_E_NOMEM, NULL);
		}
		fgetpos(p_conn->spec.mbox->file, savedpos);
		fseek(p_conn->spec.mbox->file, 0, SEEK_END);
	}
	/* No other cases require moving */

	/* Spew it out */
	fprintf(p_conn->spec.mbox->file,
			"%s\n%s", temp->header, temp->body);

	/* Cleanup */
	postal_free_msg(temp);

	/* *flush*  */
	fflush(p_conn->spec.mbox->file);

	/* If we moved our position manually, return */
	if(savedpos)
	{
		fsetpos(p_conn->spec.mbox->file, savedpos);
		free(savedpos);
	}

	/* We're good */
	return(0);
}
