/*
 * pop_connect.c
 * POP connection wrapper
 * libpostal project
 * libpost_raw  POP code
 * $Id: pop_connect.c,v 1.6 2003/06/05 11:19:31 fullermd Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_pop_p.h>
#include <post_raw_extra_p.h>


POSTAL_CONN *
pop_connect(const char *tohost, const char *user, const char *pass)
{
	POSTAL_CONN *new_connect;
	struct sockaddr_in sock_tohost;
	struct hostent *host_ret;

	/*
	 * XXX
	 * This should potentially be overridable somewhere.
	 * We should be able to set the POP port.
	 */
	sock_tohost.sin_port = htons(110);
	sock_tohost.sin_family=AF_INET;

	/*
	 * First, let's see what we got on the hostname.
	 * We'll accept both dotted decimal, and full hostname.
	 * Non-FQDN may or may not work, don't try it.
	 */
	if( (inet_aton(tohost, &sock_tohost.sin_addr)) == 0)
	{
		/* Treat as a FQDN */
		host_ret = gethostbyname(tohost);
		if(!host_ret)
			POSTAL_ERRSET_RP(POSTAL_E_NEXIST, NULL);
		
		/* Snarf */
		memcpy(&sock_tohost.sin_addr, host_ret->h_addr_list[0],
				sizeof(sock_tohost.sin_addr));
	}	


	/* Do the actual socket connection */
	new_connect = pop_network_connect(sock_tohost);
	if(!new_connect)
		POSTAL_ERRRET_RP; /* Preserve p_errno */

	/* Now, do the login */
	if( (pop_login(new_connect, user, pass)) > 0)
	{
		/* Login failed */
		/* XXX Should we try to give more information? */
		pop_network_close(new_connect);
		POSTAL_ERRRET_RP; /* Prever p_errno */
	}

	/* At this point, we should be set... */
	return(new_connect);
}
