/*
 * pop_list.c
 * POP 'LIST' query
 * libpostal project
 * libpost_raw  POP code
 * $Id: pop_list.c,v 1.20 2003/06/05 11:19:31 fullermd Exp $
 */

#include <sys/types.h>
#include <sys/uio.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_pop_p.h>
#include <post_raw_extra_p.h>


POP_RET_LIST *
pop_list(POSTAL_CONN *p_conn)
{
	char truebuf[BUFFER_SIZE+1], insertbuf[BUFFER_SIZE];
	char *buf, *buf2, *buf3, *tmpbuf=NULL, *tmpbuftmp;
	unsigned long msgbytes;
	unsigned int msgnum;
	int moretoget=1, firsttime=1, haveinsert=0;
	size_t tlen, blen=0;
	ssize_t readlen;
	POP_RET_LIST *toret;
	POP_RET_LIST *temp_head=NULL, *temp, *temp2;

	/* Send request */
	sprintf(truebuf, "LIST\r\n");
	POP_CMD_SEND(p_conn, truebuf);
	CLEARSTRING(truebuf);

	while(moretoget)
	{
		/* Get the response */
		readlen=read(p_conn->spec.pop->sock, truebuf, BUFFER_SIZE);
		if(readlen<=0)
		{
			if(temp_head)
				pop_free_retlist(temp_head);
			POSTAL_ERRSET_RP(POSTAL_E_SOCKET, NULL);
		}
		truebuf[readlen]='\0';
		buf = truebuf;

		/*
		 * Interpret the response.
		 * LIST returns a series of lines like such:
		 * +OK
		 * <msg num> <size>
		 * (repeated)
		 * with the last line containing ".\r\n"
		 */

		if(firsttime)
		{
			/* First, check that we have a good response, else bail */
			if( (strncmp(buf, POP_GOOD, strlen(POP_GOOD)) != 0) )
				POSTAL_ERRSET_RP(POSTAL_E_SRVERR, NULL);
			buf2 = strchr(buf, (int) '\r');
			*buf2 = '\0';
			buf2++;

			buf = buf2;
			firsttime=0;
		}

		/*
		 * Reassemble if we have an old partial response
		 * hanging around.
		 */
		if(haveinsert)
		{
			tlen = strlen(buf) + strlen(insertbuf) + 1;
			if(tlen>blen)
			{
				tmpbuftmp = realloc(tmpbuf, (tlen+10));
				if(tmpbuftmp==NULL)
				{
					if(temp_head)
						pop_free_retlist(temp_head);
					POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
				}
				tmpbuf=tmpbuftmp;
				blen = tlen+10;
			}
			sprintf(tmpbuf, "%s%s", insertbuf, buf);
			buf=tmpbuf;
			haveinsert=0;
		}


		/*
		 * Next, check to see if this 'batch' is the
		 * end of the LIST return.
		 * Don't look at this code, it's absurd
		 */
		buf2 = strrchr(buf, (int) '.');
		if(buf2)
		{
			if( (strncmp((buf2-1), "\n.\r\n", 4) == 0) )
			{
				/* We have it all now */
				moretoget=0;
				*buf2 = '\0';
			}
		}

		/*
		 * Now we start processing the individual items,
		 * starting at the beginning and working forward.
		 * This puts it inherently in 'highest-first'
		 * order, which is the reverse of the way we want
		 * it on the final output.
		 * Guess we'll just have to do a reversal before
		 * we return.  There should be a better way to do
		 * this.
		 * XXX FIX ME!!!
		 */
		while(*buf != '\0')
		{
			buf2 = strchr(buf, (int) '\r');
			if(!buf2)
			{
				/* We have an incomplete line */
				strcpy(insertbuf, buf);
				haveinsert=1;
				break;
			}
			else
			{
				/*
				 * XXX Hack
				 * This is necessary because of a semi-hack we
				 * used to solve a problem with a no-messages
				 * response.
				 */
				if(*buf=='\n')
					buf++;

				/* We have a full line, lets process it */
				*buf2 = '\0';
				buf2 += 1;	/* This moves us forward to the newline */

				/* Grab the message size */
				buf3 = strrchr(buf, (int) ' ');
				*buf3 = '\0';
				buf3++;
				msgbytes = strtoul(buf3, (char **)NULL, 10);

				/* And the message num */
				msgnum = atoi(buf);

				/* Pack a struct */
				temp = malloc(PRLIST_SIZE);
				if(temp==NULL)
				{
					if(temp_head)
						pop_free_retlist(temp_head);
					POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
				}
				temp->msg_num = msgnum;
				temp->msg_size = msgbytes;

				/* Link it up */
				temp->next = temp_head;
				temp_head = temp;

				/* Move us forward */
				buf = buf2;
			}
		}
	} /* Got it all */

	/* Now, we have to reverse it and return it */
	toret=NULL;			/* Just in case */
	for(temp = temp_head ; temp ; temp = temp2)
	{
		temp2 = temp->next;
		temp->next = toret;
		toret = temp;
	}

	/* Whoops, almost leaked this memory... */
	free(tmpbuf);

	/* Return the structure, we seem to be done here. */
	return(toret);
}
