/*
 * pop_network_connect.c
 * Do the TCP/IP connection to the POP server
 * libpostal project
 * libpost_raw POP code
 * $Id: pop_network_connect.c,v 1.14 2003/07/15 15:33:15 fullermd Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_pop_p.h>
#include <post_raw_extra_p.h>


POSTAL_CONN *
pop_network_connect(struct sockaddr_in server)
{
	POSTAL_CONN *new_connect;
	char *in_hdr;
	char in_hdr_tmp[POP_HDRLEN+1];
	ssize_t readlen;

	/* Get our memory ready */
	new_connect = postal_alloc_conn();
	if(new_connect==NULL)
		POSTAL_ERRRET_RP; /* Preserve p_errno */
	new_connect->spec.pop = pop_alloc_spec();
	if(new_connect->spec.pop==NULL)
	{
		free(new_connect);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	new_connect->conn_type = SRC_POP;

	/* Start our connecting... */
	new_connect->spec.pop->sock = socket(AF_INET, SOCK_STREAM, 0);
	if(new_connect->spec.pop->sock<0)
	{
		postal_free_conn(new_connect);
		POSTAL_ERRSET_RP(POSTAL_E_SOCKET, NULL);
	}
	if( connect(new_connect->spec.pop->sock, (struct sockaddr *) &server,
			sizeof(server)) < 0 ) 
	{
		close(new_connect->spec.pop->sock);
		postal_free_conn(new_connect);
		POSTAL_ERRSET_RP(POSTAL_E_SOCKET, NULL);
	}

	/* Read in the header from the POP server */
	readlen=postal_get_line(new_connect->spec.pop->sock, in_hdr_tmp, POP_HDRLEN);
	if(readlen<0)
	{
		/* We're kinda screwed, just bail and be done with it */
		pop_network_close(new_connect);
		POSTAL_ERRRET_RP; /* Preserve p_errno */
	}
	else if(readlen==0)
	{
		/* We didn't find a newline.  Forget it */
		postal_flush_line(new_connect->spec.pop->sock);
	}
	in_hdr = malloc( strlen(in_hdr_tmp)+1 );
	if(in_hdr==NULL)
	{
		pop_network_close(new_connect);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	strcpy(in_hdr, in_hdr_tmp);
	new_connect->spec.pop->srv_header = in_hdr;

	return(new_connect);
}
