/*
 * pop_retr_all.c
 * Use RETR to get all messages on server
 * libpostal project
 * libpost_raw  POP code
 * $Id: pop_retr_all.c,v 1.11 2003/07/16 10:12:40 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_pop_p.h>
#include <post_raw_extra_p.h>

#define ALLOC_CHUNK		128

POSTAL_MSG_GROUP *
pop_retr_all(POSTAL_CONN *p_conn, int dele)
{
	int mnum;
	int alloced=0;
	POP_RET_STAT *ret_stat;
	POSTAL_MSG *ret_retr, **msgstmp;
	POSTAL_MSG_GROUP *mgroup;

	mgroup = postal_alloc_msggroup();
	if(mgroup==NULL)
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	mgroup->num_msgs=0;
	mgroup->msgs=NULL;

	/* First, do a STAT to see how many messages we have */
	ret_stat = pop_stat(p_conn);
	if(ret_stat==NULL)
	{
		postal_free_msggroup(mgroup);
		POSTAL_ERRRET_RP; /* Preserve p_errno */
	}

	/*
	 * Grab the parsed messages in order, and stick them
	 * into the structure.
	 */
	for(mnum=1 ; mnum <= pop_stat_msgs(ret_stat) ; mnum++)
	{
		/* Retrieve the message */
		ret_retr = pop_retr(p_conn, mnum);
		if(ret_retr==NULL)
		{
			postal_free_msggroup(mgroup);
			POSTAL_ERRRET_RP; /* Preserve p_errno */
		}

		/* Alloc a struct and point it in */
		mgroup->num_msgs++;

		if(alloced < (mgroup->num_msgs))
		{
			/* Alloc more 'slots' */
			msgstmp = realloc( mgroup->msgs,
					(sizeof(POSTAL_MSG *) * (alloced + ALLOC_CHUNK)) );
			if(msgstmp==NULL)
			{
				postal_free_msggroup(mgroup);
				POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
			}
			mgroup->msgs = msgstmp;
			alloced += ALLOC_CHUNK;
		}
		
		(mgroup->msgs)[(mgroup->num_msgs-1)] = ret_retr;

		/* Are we DELE'ing too? */
		if(dele)
			if((pop_dele(p_conn, mnum))<0)
			{
				postal_free_msggroup(mgroup);
				POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
			}
	}

	/* Avoid a memory leak */
	free(ret_stat);

	/* Prune our memory.  Careful, Marge... */
	msgstmp = realloc( mgroup->msgs,
			(sizeof(POSTAL_MSG *) * (mgroup->num_msgs)) );
	if(msgstmp==NULL)
	{
		postal_free_msggroup(mgroup);
		POSTAL_ERRSET_RP(POSTAL_E_NOMEM, NULL);
	}
	mgroup->msgs = msgstmp;

	/* That done, just return the puppy */
	return(mgroup);
}
