/*
 * listfrom.c
 * Test mbox reading functions
 * Specially designed to work in the compile tree
 * Links again: libpost_raw
 * Links shared
 * $Id: listfrom.c,v 1.30 2003/07/16 10:12:40 fullermd Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "../../include/post_raw/post_raw.h"

#ifdef USE_DMALLOC
#include <dmalloc.h>
#endif

#define	ALLOC_CHUNK		128

int
main(int argc, char *argv[])
{
	POSTAL_CONN *my_conn;
	POSTAL_MSG_GROUP *msgs;
	POSTAL_MSG *msgtmp;
	long i;
	char *target;
	char *ret_hdr, *ret_hdr2;
	char modes[256];
	int msgstat;
	int goch;
	int quiet=0, iterate=0, body=0;
	char stats[128]; /* Wasteful.  Who cares. */
	struct stat st_ret;
	int maildir=0;

	/*
	 * Command-line args:
	 * -q: Quiet mode - no output, useful for speed-tests
	 * -i: Iterate - Call mbox_get_message_next() repeatedly, instead of
	 *     using mbox_get_message_all()
	 */
	while( (goch = getopt(argc, argv, "qib")) != -1 )
		switch((char)goch)
		{
			case 'q':
				/* Quiet mode */
				quiet=1;
				break;
			case 'i':
				/* Iterative mode */
				iterate=1;
				break;
			case 'b':
				/* GET_BODY */
				body=1;
				break;
			default:
				fprintf(stderr, "Bad argument: '%c'\n", goch);
				exit(1);
				/* NOTREACHED */
		}
	
	argc -= optind;
	argv += optind;

	/* We better have a mailbox name, or bad things will happen */
	if(argc<1)
	{
		fprintf(stderr, "Need mailbox name, damn you!\n");
		exit(1);
	}
	target = strdup(*argv);

	/* If it's a directory, play Maildir for me */
	if( stat(target, &st_ret) < 0 )
	{
		fprintf(stderr, "File/directory '%s' not found!\n", target);
		exit(1);
	}
	if(st_ret.st_mode & S_IFDIR)
		maildir=1;

	/* Error-check */
	if( (maildir==1) && iterate )
	{
		fprintf(stderr, "-i on a maildir makes no sense\n");
		exit(1);
	}

	/* Inform the luser what we're going to do */
	modes[0]='\0';
	if(quiet)
		strcat(modes, "quiet ");
	if(iterate)
		strcat(modes, "iterative ");
	if(body)
		strcat(modes, "body-fetch ");

	printf("Running on %s '%s'%s%s%s\n",
			(maildir ? "maildir" : "mbox"),
			target,
			((quiet||body||iterate) ? " in " : ""),
			modes,
			((quiet||body||iterate) ? "mode" : ""));

	if(maildir==0)
	{
		/* It's a mbox */
		my_conn=mbox_open_file(target, LOCK_DEFAULT, OPEN_READONLY);

		if(my_conn==NULL)
		{
			fprintf(stderr, "Error opening mbox '%s'.  Oopsie.\n", target);
			fprintf(stderr, "Error: %s\n", postal_strerror(postal_errno()));
			exit(1);
		}

		/*
		 * Get the stuff.
		 * We have the option of either using mbox_get_message_all(), or
		 * doing our own manual iteration using mbox_get_message_next().
		 * You'd never really want to do the latter in a real program,
		 * but we include it here for testing and benchmarking purposes.
		 */ 
		if(iterate==0)
		{
			/* Use get_all.  Gee, this is easy  :) */
			if((mbox_get_message_all(my_conn, (body? GET_BODY : GET_HEADER)))
					< 0)
			{
				mbox_close_file(my_conn);
				fprintf(stderr, "Error in mbox_get_message_all(): %s\n",
						postal_strerror(postal_errno()));
				exit(1);
			}

			msgs = postal_mgroup(my_conn);
			if(msgs==NULL)
			{
				mbox_close_file(my_conn);
				fprintf(stderr, "Got no messages, that's kinda weird.\n");
				exit(1);
			}
		}
		else
		{
			/* Recurse ourselves.  Gee, this is a PITA */
			my_conn->msgs=NULL; /* We shouldn't have to do this manually */
			msgs = postal_alloc_msggroup();

			/* We need to start at the beginning of the file of course */
			lseek(my_conn->spec.mbox->desc, 0, SEEK_SET);

			/* Read in a message at a time until we're done */
			while( (msgtmp = mbox_get_message_next(my_conn,
					(body ? GET_BODY : GET_HEADER) )) )
			{
				/* XXX Need to error-check here */
				msgs->num_msgs++;

				if(msgs->alloced < (msgs->num_msgs))
				{
					/*
					 * We need to allocate more space.
					 * We're doing it in chunks of ALLOC_CHUNK.
					 */
					msgs->msgs = realloc( msgs->msgs,
							(sizeof(POSTAL_MSG *)
							* (msgs->alloced + ALLOC_CHUNK)) );
					msgs->alloced += ALLOC_CHUNK;
				}

				/* We've got a message, add it in */
				(msgs->msgs)[(msgs->num_msgs-1)] = msgtmp;
				msgs->alloced = msgs->num_msgs;
			}

			/* Prune the memory.  This is a bit dicey. */
			msgs->msgs = realloc( msgs->msgs,
					(sizeof(POSTAL_MSG *) * (msgs->num_msgs)) );
		}
	}
	else
	{
		/* It's a Maildir */
		my_conn=maildir_open(target, 0);

		if(!my_conn)
		{
			fprintf(stderr, "Error opening maildir '%s'.  Oopsie.\n", target);
			fprintf(stderr, "Error: %s\n", postal_strerror(postal_errno()));
			exit(1);
		}

		if( (maildir_get_message_all(my_conn, (body? GET_BODY : GET_HEADER)))
				< 0)
		{
			maildir_close(my_conn);
			fprintf(stderr, "Error in maildir_get_message_all(): %s\n",
					postal_strerror(postal_errno()));
			exit(1);
		}
		msgs = postal_mgroup(my_conn);
		if(msgs==NULL)
		{
			maildir_close(my_conn);
			fprintf(stderr, "Got no messages, that's kinda weird.\n");
			exit(1);
		}
	}
	free(target);

	printf("Read %li messages\n", postal_mgroup_num(msgs));

	if(quiet==0)
	{
		/* For each message, list msg #, size, status, and From: header */
		for(i=1 ; i <= postal_mgroup_num(msgs) ; i++)
		{
			msgtmp = postal_mgroup_getmsg(msgs, i);
			ret_hdr = postal_get_header(msgtmp->header, "From:");
			ret_hdr2 = postal_get_header(msgtmp->header, "Message-Id:");
			msgstat = postal_msg_mbstat(msgtmp);
			bzero(stats, 128);
			if(msgstat&STATUS_NEW)
				strcat(stats, "New ");
			if(msgstat&STATUS_OLD)
				strcat(stats, "Old ");
			if(msgstat&STATUS_READ)
				strcat(stats, "Read ");
			if(msgstat&STATUS_REPLIED)
				strcat(stats, "Answered ");
			if(strlen(stats)==0)
				strcat(stats, "(none)");
			if(maildir==0)
				printf("#%-5li %4i/%6i hdr/body, Status: %s, Offset: %li\n"
						"\t\tFrom: '%s'\n"
						"\t\tMessage-Id: '%s'\n",
						i, postal_msg_hsize(msgtmp),
						postal_msg_bsize(msgtmp), stats,
						(long)msgtmp->loc.offset,
						ret_hdr, ret_hdr2);
			else
				printf("#%-5li %4i/%6i hdr/body, Status: %s\n"
						"\t\tFilename: %s\n"
						"\t\tFrom: '%s'\n"
						"\t\tMessage-Id: '%s'\n",
						i, postal_msg_hsize(msgtmp),
						postal_msg_bsize(msgtmp), stats,
						msgtmp->loc.file->filename,
						ret_hdr, ret_hdr2);
			free(ret_hdr);
			free(ret_hdr2);
		}
	}

	/*
	 * If we're in iterative mode, we built the POSTAL_MSG_GROUP
	 * ourselves, so free() it ourselves.  If we ran in 'normal' mode, it
	 * was built as part of mbox_get_message_all(), which means it's a
	 * part of the POSTAL_CONN structure, so don't free it manually.
	 */
	if(iterate==1)
		postal_free_msggroup(msgs);

	/* This free()'s ->msgs if not-null */
	if(maildir==0)
		mbox_close_file(my_conn);
	else
		maildir_close(my_conn);
	exit(0);
}
