#!/usr/bin/perl -w
# makemakefiles.pl
# Written for the libpostal project
# $Id: mkmakefiles.pl,v 1.13 2003/05/26 08:29:28 fullermd Exp $

# Make bmake-style Makefiles and gmake-style GNUmakefiles
# from a .proto makefile.
# This is severely ugly, and inefficient to boot.  Too bad.
# You think you can do it better, then *PLEASE* do.

use Cwd;

# First argument is the path to the base of the libpostal tree
$lpdir = $ARGV[0];
chomp $lpdir;

# Second is the .proto Makefile
$protofile = $ARGV[1];
chomp $protofile;

# Figure out what our makefiles are
# Don't try this at home, kids...
$bname = $protofile;
$bname =~ s/\.proto$//;
$bmakefile = $gmakefile = $bname;
$gmakefile =~ s/Makefile/GNUmakefile/;

# Now lets get conversions going...
# Our input
open(PROTO, "$protofile") or die "Can't open input file '$protofile'!";
open(BMAKE, ">$bmakefile") or die "Can't open outfile file '$bmakefile'!";
open(GMAKE, ">$gmakefile") or die "Can't open outfile file '$gmakefile'!";
print BMAKE "# DO NOT EDIT (see $protofile)";
print GMAKE "# DO NOT EDIT (see $protofile)";
while(<PROTO>)
{
	chomp;
	$inline = $_;
	# Now the fun begins
	# Chop out all the comment lines...
	next if $inline =~ /^#/;

	# Clean more comment stuff
	$inline =~ s/#.*$//;

	# Now, let's see if it's one of our 'special' tags
	if( $inline =~ /^MKVARDEFS/ )
	{
		# Put in the variable definitions that we want...
		unlink("${lpdir}/tools/defs\.out");
		open(VDEF, "$lpdir/tools/progdefs");
		while(<VDEF>)
		{
			# Holy crap, this is ugly.  Oh well.
			chomp;
			# Ignore comment line
			if(/^#/)
			{	next;	}
			($vname, $pname) = split /\=/;
			system("printf \"$vname =\" >> $lpdir/tools/defs.out");
			system("$lpdir/tools/findprog.sh $pname $lpdir/tools >> $lpdir/tools/defs.out");
		}
		close VDEF;
		open(DOUT, "$lpdir/tools/defs\.out");
		while(<DOUT>)
		{
			print BMAKE $_;
			print GMAKE $_;
		}
		close DOUT;
		unlink("$lpdir/tools/defs.out");

		next;
	}

	if( $inline =~ /^ADD_DEPEND/ )
	{
		# Add the dependancy include in the GNU makefile
		print GMAKE "-include \${.CURDIR}/.depend\n";
		next;
	}

	if( $inline =~ /^CURDIR_SUB/ )
	{
		# Yet another thing we only have to do for GNU Makefiles...
		print GMAKE ".CURDIR = \${PWD}\n";
		next;
	}

	# Now, start the subs
	$bmake = $gmake = $inline;

	# INCLUDE()
	$bmake =~ s/INCLUDE\(([^\)]+)\)/\.include\ \"$1\"/;
	$gmake =~ s/INCLUDE\(([^\)]+)\)/include\ $1/;
	$gmake =~ s/Makefile/GNUmakefile/;

	# NOTDIR
	$bmake =~ s/NOTDIR\(([^\)]+)\}\)/$1:T\}/;
	$gmake =~ s/NOTDIR\(([^\)]+)\}\)/\$\(notdir\ $1\}\)/;

	# A few variable usages
	$gmake =~ s/\.TARGET/\@/g;
	$gmake =~ s/\.IMPSRC/\</g;

	# Conditionals
	$gmake =~ s/^\.(ifn?def)\ \(([A-Z_]+)\)/$1\ $2/;
	$gmake =~ s/^\.([ei]+)/$1/;

	# Write out the line
	print BMAKE "$bmake\n";
	print GMAKE "$gmake\n";
	# Fin
}
close(PROTO);
close(BMAKE);
close(GMAKE);
